/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \page lcas_timeslots.mod time slots plugin

    \section timeslotssyn SYNOPSIS
        \b lcas_timeslots.mod [\<time slots file\>]

    \section timeslotsdescr DESCRIPTION

    \author Martijn Steenbakkers for the EU DataGrid.

        This plugin makes an authorization decisions based on available time slots.
        Currently it reads a text file that contains the available time slots.

    \section timeslotsoptions OPTIONS
    \subsection timeslotsoptie1 [<time slots file>]
        The name of the file that contains the list of time slots.
        Default search path: \c /opt/edg/lib/lcas/modules.
        The format of the file is described here:
\verbatim
#
# This file contains the time slots for which the fabric
# is available for Grid jobs
# Format:
#        minute1-minute2 hour1-hour2 mday1-mday2 month1-month2 year1-year2 wday1-wday2
# max range:  [0-59]        [0-23]      [1-31]       [1-12]    [1970-...]     [0-6]
#
# wday:
#  0-6 = Sunday-Saturday
#  5-3 = Friday-Wednesday
#
# '*' means the maximum range
# <val>- means from <val> to maximum value
#
# The wall clock time should match at least one time slot for authorization
# The wall clock time matches if:
#           (hour1:minute1)      <= (hour:minute)     <= (hour2:minute2) 
#       AND (year1.month1.mday1) <= (year.month.mday) <= (year2.month2.mday2)
#       AND (wday1)              <= (wday)            <= (wday2)
#
# If the fabric is open on working days from 8:30-18:00 h, from 1 July 2002 till 15 January 2003
# the following line should be uncommented:
#      30-0           8-18        1-15         7-1       2002-2003      1-5
#
# If the fabric is open from 18:00-7:00 h, two time slots should be used:
#      18:00-24:00 and 0:00-7:00
#      
#       0-0          18-24         *            *            *           *
#       0-0           0-7          *            *            *           *
# If the fabric is always open the following line should be uncommented:
#         *             *           *            *            *           *
\endverbatim


\section timeslotsReturnvalue RETURN VALUES
        \li LCAS_MOD_SUCCESS : Success
        \li LCAS_MOD_NOFILE  : the time slots file could not be found
        \li LCAS_MOD_FAIL    : Failure
 
\section timeslotsErrors ERRORS
        See bugzilla for known errors (http://marianne.in2p3.fr/datagrid/bugzilla/)
 
\section timeslotsSeeAlso SEE ALSO
        \ref lcas_userallow.mod "lcas_userallow.mod",
        \ref lcas_userban.mod "lcas_userban.mod",
        \ref lcas_voms.mod "lcas_voms.mod",
*/

/*!
    \file   lcas_timeslots.c
    \brief  Interface to the LCAS plugins
    \author Martijn Steenbakkers for the EU DataGrid.

    LCAS plugin that makes authorization decisions based on available time slots
    Currently it reads a text file that contains the available time slots.
    -# plugin_initialize()
    -# plugin_confirm_authorization()
    -# plugin_terminate()
*/

/******************************************************************************

lcas_timeslots.c

Description:
    LCAS module that makes authorization decisions based on available time slots
    Currently it reads a text file that contains the available time slots.

CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/lcas-plugins-basic/src/timeslots/lcas_timeslots.c,v $
    $Date: 2010-05-03 10:44:00 $
    $Revision: 1.4 $
    $Author: okoeroo $

******************************************************************************/

/*****************************************************************************
                            Include header files
******************************************************************************/
#include "lcas_config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if HAVE_MALLOC_H
#include <malloc.h>
#endif

#if HAVE_TIME_H
#include <time.h>
#endif

#if HAVE_VALUES_H
#include <values.h>
#endif

#include <lcas/lcas_modules.h>

/******************************************************************************
                                Definitions
******************************************************************************/

/*
 * Definitions for fabric wallclock check
 */
#define LINELEN         65536
#define FIELDLEN        10
#define MAXSLOTS        100
#define NFIELDS         6
#define MINUTE_FIELD    0
#define HOUR_FIELD      1
#define MDAY_FIELD      2
#define MONTH_FIELD     3
#define YEAR_FIELD      4
#define WDAY_FIELD      5
#define WDAY_MIN        0
#define WDAY_MAX        7

#define HOUR_FAIL       (unsigned short) 0x0
#define HOUR_OK         (unsigned short) 0x1
#define WDAY_FAIL       (unsigned short) 0x0
#define WDAY_OK         (unsigned short) 0x2
#define DATE_FAIL       (unsigned short) 0x0
#define DATE_OK         (unsigned short) 0x4
#define ALL_FAIL        (unsigned short) 0x0
#define ALL_OK          (unsigned short) 0x7

/******************************************************************************
                          Module specific prototypes
******************************************************************************/
static int            findrange(char * range_string, int * minval, int * maxval);
static int            get_timeslots(char * timeslots_file);
static unsigned short check_date(time_t clock, struct tm * pstart, struct tm * pstop);
static unsigned short check_wday(time_t clock, struct tm * pstart, struct tm * pstop);
static unsigned short check_hour(time_t clock, struct tm * pstart, struct tm * pstop);

/******************************************************************************
                       Define module specific variables
******************************************************************************/
/*
 * Variables for fabric wallclock check
 */
static int                    nslots;
static struct tm              start_s[MAXSLOTS];
static struct tm              stop_s[MAXSLOTS];
static char *                 modname="lcas_timeslots.mod";
static char *                 clockcheck_db = NULL;
static char *                 days[7]=
                              {
                                  "Sunday",
                                  "Monday",
                                  "Tuesday",
                                  "Wednesday",
                                  "Thursday",
                                  "Friday",
                                  "Saturday"
                              };
static char *                 months[12]=
                              {
                                  "Jan",
                                  "Feb",
                                  "Mar",
                                  "Apr",
                                  "May",
                                  "Jun",
                                  "Jul",
                                  "Aug",
                                  "Sep",
                                  "Oct",
                                  "Nov",
                                  "Dec",
                              };
/******************************************************************************
Function:   plugin_initialize
Description:
    Initialize plugin
Parameters:
    argc, argv
    argv[1]: database to be used by plugin
Returns:
    LCAS_MOD_SUCCESS : succes
    LCAS_MOD_FAIL    : failure
******************************************************************************/
#if 0
int plugin_initialize(char * plugin_db)
#endif
int plugin_initialize(int argc, char ** argv)
{
    int                           i;

    lcas_log_debug(1,"%s-plugin_initialize(): passed arguments:\n",modname);
    for (i=0; i < argc; i++)
    {
        lcas_log_debug(1,"%s-plugin_initialize(): arg %d is %s\n",
             modname,i,argv[i]);
    }

    if (argc > 1)
        clockcheck_db = lcas_findfile(argv[1]);

    /* Test if clockcheck_db can be opened */
    if (clockcheck_db == NULL)
    {
        lcas_log(0,"\t%s-plugin_initialize() error: timeslots file required !\n",
                 modname);
        return LCAS_MOD_NOFILE;
    }
    if (lcas_getfexist(1,clockcheck_db) == NULL)
    {
        lcas_log(0,
            "\t%s-plugin_initialize() error: Cannot find fabric availability (time slots) file: %s\n",
            modname,clockcheck_db);
        return LCAS_MOD_NOFILE;
    }

    /* Fill tm structures "start_s" and "stop_s" */
    lcas_log_debug(1,"\t%s-plugin_initialize(): looking for timeslots in %s\n",
             modname,clockcheck_db);
    nslots=get_timeslots(clockcheck_db);
    if (nslots < 0)
    {
        lcas_log(0,"\t%s-plugin_initialize(): Cannot read fabric availability file\n",
                 modname);
        return LCAS_MOD_FAIL;
    }
    else if (nslots==0)
    {
        /* No slots found == fabric is closed */
        lcas_log(0,"\t%s-plugin_initialize(): empty fabric availability file\n",
                 modname);
    }
    
    return LCAS_MOD_SUCCESS;
}

/******************************************************************************
Function:   plugin_confirm_authorization
Description:
    Ask for authorization by passing RSL and user credential
Parameters:
    request:   RSL request
    user_cred: user credential
Returns:
    LCAS_MOD_SUCCESS : succes
    LCAS_MOD_FAIL    : failure
******************************************************************************/
int
plugin_confirm_authorization(lcas_request_t request, lcas_cred_id_t lcas_cred)
{
    time_t                        clock;
    struct tm *                   clock_s;
    int                           passflag;
    int                           islot;
    unsigned short                slotflag;

    /* Get the local time */
    /*     future: might be feeded through function call */
    clock=time(NULL);
    clock_s = localtime(&clock);

    /*
     *  Loop over the different slots
     *
     *  Change in policy:
     *       1- Check dates
     *       2- Check day of the week
     *       3- Check hour,minute(,second)
     *
     *  Note: For simplicity start values should always be smaller
     *  than stop values, e.g. if you want to open the fabric during
     *  night from 20.00 to 8.00, you have to specify both 20-24 and 0-8.
     *       
     */

    passflag=0;
    for (islot=0; islot < nslots; ++islot)
    {
        slotflag=ALL_FAIL;
        lcas_log_debug(0,"\t%s-plugin_confirm_authorization(): Checking slot %d out of %d in %s\n",
                 modname,islot+1,nslots,clockcheck_db);

        /* Check date */
        slotflag|=check_date(clock,start_s+islot,stop_s+islot);
        
        /* Check week day */
        slotflag|=check_wday(clock,start_s+islot,stop_s+islot);

        /* Check hour */
        slotflag|=check_hour(clock,start_s+islot,stop_s+islot);

        if (slotflag==ALL_OK) passflag=1;
    }

    if (passflag!=1) return LCAS_MOD_FAIL;

    return LCAS_MOD_SUCCESS;
}

/******************************************************************************
Function:   plugin_terminate
Description:
    Terminate plugin
Parameters:

Returns:
    LCAS_MOD_SUCCESS : succes
    LCAS_MOD_FAIL    : failure
******************************************************************************/
int plugin_terminate()
{
    lcas_log_debug(1,"%s-plugin_terminate(): terminating\n",modname);
    if (clockcheck_db) { free(clockcheck_db); clockcheck_db=NULL; }

    return LCAS_MOD_SUCCESS;
}

/******************************************************************************
Function:   findrange
Description:
    Convert a given string to an integer range
Parameters:
    range_string : time value to be checked
    minval       : extracted minimum integer value
    maxval       : extracted maximum integer value
Returns:
    0 : succes
    1 : failure
******************************************************************************/
static int
findrange(char * range_string, int * minval, int * maxval)
{
    char *                        t=NULL;
    char *                        u=NULL;

    if (range_string==NULL) return 1;

    /*
     * string = '*' : maximum range
     */
    if (strcmp(range_string,"*")==0)
    {
        *minval=*maxval=-1;
        return 0;
    };

    /*
     * string = '-....' : minimum - .....
     */
    if (strncmp(range_string,"-",1)==0)
    {
        *minval=-1;
        if (strlen(range_string)>1)
        {
            if ( (t=strtok(range_string,"-")) )
            {
                *maxval=strtol(t,&u,0);
                if(t==u) {
                    lcas_log(0,"\t%s-findrange(): Cannot read long integer from string %s\n",
                             modname,t);
                    return 1;
                }
            }
            else
            {
                lcas_log(0,"\t%s-findrange(): Cannot separate string %s\n",
                         modname,range_string);
                return 1;
            }
        }
        else
        {
            /*
             * string = '-' : maximum range
             */
            *maxval=-1;
        }
        return 0;
    }

    /*
     * string = '....-....'
     * read first argument
     */
    if ( (t=strtok(range_string,"-")) )
    {
        *minval=strtol(t,&u,0);
        if(t==u) {
            lcas_log(0,"\t%s-findrange(): Cannot read long integer from string %s\n",
                     modname,t);
            return 1;
        }
    }
    else
    {
        lcas_log(0,"\t%s-findrange(): Cannot separate string %s\n",
                 modname,range_string);
        return 1;
    }

    /*
     * read second argument
     */
    if ( (t=strtok(NULL,"-")) )
    {
        *maxval=strtol(t,&u,0);
        if(t==u) {
            lcas_log(0,"\t%s-findrange(): Cannot read long integer from string %s\n",
                     modname,t);
            return 1;
        }
    }
    /*
     * string = '....-'
     */
    else
    {
        *maxval=-1;
    }
    return 0;
}

/******************************************************************************
Function:   get_timeslots
Description:
    Get the fabric availability parameters 
    Currently the parameters are read from a (cron style) file

Parameters:
    timeslots_file: text database containing the time slots

Returns:
    number of time slots found in file
******************************************************************************/
static int
get_timeslots(char * timeslots_file)
{
    time_t                        clock;
    struct tm *                   clock_s;
    FILE *                        fp;
    char                          lbuf[LINELEN];
    char *                        s;
    int                           iline,ifield;
    int                           islot;
    int                           rangemin[NFIELDS],rangemax[NFIELDS];
    char *                        fields[NFIELDS];

    time(&clock);
    clock_s=localtime(&clock);

    /*
     * Open times file
     * Format:
     *   minute1-minute2 hour1-hour2 mday1-mday2 month1-month2 year1-year2 wday1-wday2
     *        [0-59]        [0-23]      [1-31]       [1-12]    [1970-...]     [0-6]
     *
     * wday: 0-6 = Sunday-Saturday
     *
     * for '*' and '-' the maximum range is substituted
     *
     */
    fp=fopen(timeslots_file,"r");
    if( fp == (FILE *)0)
    {
        lcas_log(0,"\t%s-get_timeslots(): FILE %s could not be opened for input\n",
                 modname,timeslots_file);
        return -1;
    }
    iline=0;
    islot=0;
    while(fgets(lbuf,LINELEN,fp))
    {
        /* Parse line */
        if (lbuf[0]!='#') /* Skip comments */
        {
            if (islot >= MAXSLOTS)
            {
                lcas_log(0,"\t%s-get_timeslots() error: Too many time slots found (max=%d)\n",
                         modname,MAXSLOTS);
                return -MAXSLOTS;
            }
            /*
             * Initialize start_s and stop_s with some reasonable values
             */
            start_s[islot].tm_sec=0;               stop_s[islot].tm_sec=0;
            start_s[islot].tm_min=0;               stop_s[islot].tm_min=59;
            start_s[islot].tm_hour=0;              stop_s[islot].tm_hour=23;
            start_s[islot].tm_mday=1;              stop_s[islot].tm_mday=31;
            start_s[islot].tm_mon=0;               stop_s[islot].tm_mon=11;
            start_s[islot].tm_year=70;             stop_s[islot].tm_year=137;
            start_s[islot].tm_wday=0;              stop_s[islot].tm_wday=6;
            start_s[islot].tm_yday=0;              stop_s[islot].tm_yday=364;
            start_s[islot].tm_isdst=clock_s->tm_isdst; stop_s[islot].tm_isdst=clock_s->tm_isdst;

            /* Separate the different fields and put the pointers in "fields" */
            ifield=0;
            lcas_log_debug(4,"\t%s-get_timeslots(): buffer: %s\n",modname,lbuf);
            s=strtok(lbuf," \t\n");
            if (strlen(s) > FIELDLEN)
            {
                lcas_log(0,"\t%s-get_timeslots(): field %d: %s is too long (MAX=%d)\n",
                         modname,ifield,s,FIELDLEN);
                return -1;
            }
            fields[ifield]=s;
            lcas_log_debug(4,"\t%s-get_timeslots(): ifield: %d, field: %s\n",modname,ifield,fields[ifield]);
            for(++ifield;(s=strtok(NULL," \t\n")) && (ifield < NFIELDS) ;++ifield)
            {
                if (strlen(s) > FIELDLEN)
                {
                    lcas_log(0,"\t%s-get_timeslots(): field %d: %s is too long (MAX=%d)\n",
                             modname,ifield,s,FIELDLEN);
                    return -1;
                }
                fields[ifield]=s;
                lcas_log_debug(4,"\t%s-get_timeslots(): ifield: %d, field: %s\n",modname,ifield,fields[ifield]);
            }
            if (ifield != NFIELDS)
            {
                lcas_log(0,"\t%s-get_timeslots(): Should have found %d fields, but found %d fields instead\n",
                    modname,NFIELDS,ifield);
                return -1;
            }

            /* split the fields and put the values in rangemin and rangemax */
            for(ifield=0;(ifield < NFIELDS) ;++ifield)
            {
                if (findrange(fields[ifield],&rangemin[ifield],&rangemax[ifield]))
                {
                    lcas_log(0,
                        "\t%s-get_timeslots(): Cannot extract time range of field %d in FILE %s\n",
                        modname,ifield,timeslots_file);
                    return -1;
                }
                lcas_log_debug(4,"\t%s-get_timeslots(): ifield: %d, rangemin-rangemax: %d - %d\n",modname,ifield,rangemin[ifield],rangemax[ifield]);
                switch (ifield)
                {
                    case MINUTE_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_min=rangemin[ifield];
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_min=rangemax[ifield];
                        break;
                    case HOUR_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_hour=rangemin[ifield];
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_hour=rangemax[ifield];
                        break;
                    case MDAY_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_mday=rangemin[ifield];
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_mday=rangemax[ifield];
                        break;
                    case MONTH_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_mon=rangemin[ifield]-1;
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_mon=rangemax[ifield]-1;
                        break;
                    case YEAR_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_year=rangemin[ifield]-1900;
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_year=rangemax[ifield]-1900;
                        break;
                    case WDAY_FIELD:
                        if (rangemin[ifield] >= 0) start_s[islot].tm_wday=rangemin[ifield];
                        if (rangemax[ifield] >= 0) stop_s[islot].tm_wday=rangemax[ifield];
                        break;
                    default:
                        lcas_log_debug(0,"\t%s-get_timeslots(): fieldnr: %d is not used\n",
                        modname,ifield);
                        break;
                }
            }
            islot++;
        }
        iline++;
    }
    fclose(fp);

    return islot;
}

/******************************************************************************
Function:   check_date
Description:
    Check if a time value matches the dates of a begin and an end time structure
    of a timeslot
Parameters:
    clock:  time value to be checked
    pstart: pointer to time structure of start of time slot
    pstop:  pointer to time structure of stop of time slot
Returns:
    DATE_OK  : time value matches the date range of a time slot
    DATE_FAIL: time value does not match the date range of a time slot
******************************************************************************/
static unsigned short
check_date(time_t clock, struct tm * pstart, struct tm * pstop)
{
    time_t                        date1,date2;
    struct tm *                   date1_s = (struct tm *) malloc(sizeof(struct tm));
    struct tm *                   date2_s = (struct tm *) malloc(sizeof(struct tm));
    struct tm *                   clock_s;

    clock_s = localtime(&clock);

    date1_s->tm_sec   = date2_s->tm_sec   = clock_s->tm_sec;
    date1_s->tm_min   = date2_s->tm_min   = clock_s->tm_min;
    date1_s->tm_hour  = date2_s->tm_hour  = clock_s->tm_hour;
    date1_s->tm_mday  = date2_s->tm_mday  = clock_s->tm_mday;
    date1_s->tm_mon   = date2_s->tm_mon   = clock_s->tm_mon;
    date1_s->tm_year  = date2_s->tm_year  = clock_s->tm_year;
    date1_s->tm_wday  = date2_s->tm_wday  = clock_s->tm_wday;
    date1_s->tm_yday  = date2_s->tm_yday  = clock_s->tm_yday;
    date1_s->tm_isdst = date2_s->tm_isdst = clock_s->tm_isdst;

    date1_s->tm_mday  = pstart->tm_mday;
    date1_s->tm_mon   = pstart->tm_mon;
    date1_s->tm_year  = pstart->tm_year;
    date2_s->tm_mday  = pstop->tm_mday;
    date2_s->tm_mon   = pstop->tm_mon;
    date2_s->tm_year  = pstop->tm_year;

    date1=mktime(date1_s);
    date2=mktime(date2_s);

    if (difftime(date2,date1) < 0.0)
    {
        lcas_log_debug(0,"\t%s-check_date():     Wrong dates: start date is later than stop date\n",
                 modname);
        free((void *) date1_s);
        free((void *) date2_s);
        return DATE_FAIL;
    }
    else if ( (difftime(date2,clock) < 0.0) ||
              (difftime(clock,date1) < 0.0) )
    {
        lcas_log(0,"\t%s-check_date():     Date (%d %s %d) out of range: (%d %s %d)-(%d %s %d)\n",
            modname,
            clock_s->tm_mday,
            months[clock_s->tm_mon],
            clock_s->tm_year+1900,
            date1_s->tm_mday,
            months[date1_s->tm_mon],
            date1_s->tm_year+1900,
            date2_s->tm_mday,
            months[date2_s->tm_mon],
            date2_s->tm_year+1900
        );
        free((void *) date1_s);
        free((void *) date2_s);
        return DATE_FAIL;
    }

    free((void *) date1_s);
    free((void *) date2_s);
    return DATE_OK;
}

/******************************************************************************
Function:   check_wday
Description:
    Check if a time value matches the week days of a begin and an end time structure
    of a timeslot
Parameters:
    clock:  time value to be checked
    pstart: pointer to time structure of start of time slot
    pstop:  pointer to time structure of stop of time slot
Returns:
    WDAY_OK  : time value matches the week day range of a time slot
    WDAY_FAIL: time value does not match the week day range of a time slot
******************************************************************************/
static unsigned short
check_wday(time_t clock, struct tm * pstart, struct tm * pstop)
{
    struct tm *                   clock_s;
    int                           start_wday,stop_wday,wday;

    start_wday=pstart->tm_wday;
    stop_wday=pstop->tm_wday;

    /* Check start_wday value */
    if ( (start_wday < WDAY_MIN) || (start_wday > WDAY_MAX) )
    {
        lcas_log_debug(0,"\t%s-check_wday():     Start week day (%d) out of range (%d-%d)\n",
                 modname,start_wday,WDAY_MIN,WDAY_MAX);
        return WDAY_FAIL;
    }
    else if (start_wday == WDAY_MAX)
    {
        start_wday=WDAY_MIN;
    }

    /* Check stop_wday value */
    if ( (stop_wday < WDAY_MIN) || (stop_wday > WDAY_MAX) )
    {
        lcas_log_debug(0,"\t%s-check_wday():     Stop week day (%d) out of range (%d-%d)\n",
                 modname,stop_wday,WDAY_MIN,WDAY_MAX);
        return WDAY_FAIL;
    }
    else if (stop_wday == WDAY_MAX)
    {
        stop_wday=WDAY_MIN;
    }
    
    /* Get wday value */
    clock_s = localtime(&clock);
    wday=clock_s->tm_wday;
    if (wday == WDAY_MAX) wday=WDAY_MIN;

    /* Check if wday is in range */
    if (stop_wday >= start_wday)
    {
        if ( (wday > stop_wday) || (wday < start_wday) )
        {
            lcas_log_debug(0,"\t%s-check_wday():     Wday (%s) out of range (%s-%s)\n",
                     modname,days[wday],days[start_wday],days[stop_wday]);
            return WDAY_FAIL;
        }
    }
    else
    {
        stop_wday+=WDAY_MAX;
        if ( (wday > stop_wday) || (wday < start_wday) )
        {
            wday+=WDAY_MAX;
            if ( (wday > stop_wday) || (wday < start_wday) )
            {
                lcas_log_debug(0,"\t%s-check_wday():     Wday (%s) out of range (%s-%s)\n",
                         modname,
                         days[wday-WDAY_MAX],days[start_wday],
                         days[stop_wday-WDAY_MAX]);
                return WDAY_FAIL;
            }
        }
    }

    return WDAY_OK;
}

/******************************************************************************
Function:   check_hour
Description:
    Check if a time value matches the hours of a begin and an end time structure
    of a timeslot
Parameters:
    clock:  time value to be checked
    pstart: pointer to time structure of start of time slot
    pstop:  pointer to time structure of stop of time slot
Returns:
    HOUR_OK  : time value matches the hour range of a time slot
    HOUR_FAIL: time value does not match the hour range of a time slot
******************************************************************************/
static unsigned short
check_hour(time_t clock, struct tm * pstart, struct tm * pstop)
{
    time_t                        date1,date2;
    struct tm *                   date1_s = (struct tm *) malloc(sizeof(struct tm));
    struct tm *                   date2_s = (struct tm *) malloc(sizeof(struct tm));
    struct tm *                   clock_s;

    clock_s = localtime(&clock);

    date1_s->tm_sec   = date2_s->tm_sec   = clock_s->tm_sec;
    date1_s->tm_min   = date2_s->tm_min   = clock_s->tm_min;
    date1_s->tm_hour  = date2_s->tm_hour  = clock_s->tm_hour;
    date1_s->tm_mday  = date2_s->tm_mday  = clock_s->tm_mday;
    date1_s->tm_mon   = date2_s->tm_mon   = clock_s->tm_mon;
    date1_s->tm_year  = date2_s->tm_year  = clock_s->tm_year;
    date1_s->tm_wday  = date2_s->tm_wday  = clock_s->tm_wday;
    date1_s->tm_yday  = date2_s->tm_yday  = clock_s->tm_yday;
    date1_s->tm_isdst = date2_s->tm_isdst = clock_s->tm_isdst;

    date1_s->tm_sec   = pstart->tm_sec;
    date1_s->tm_min   = pstart->tm_min;
    date1_s->tm_hour  = pstart->tm_hour;
    date2_s->tm_sec   = pstop->tm_sec;
    date2_s->tm_min   = pstop->tm_min;
    date2_s->tm_hour  = pstop->tm_hour;

    date1=mktime(date1_s);
    date2=mktime(date2_s);
    if (date2_s->tm_hour==0) date2_s->tm_hour=24;

    if (difftime(date2,date1) < 0.0)
    {
        lcas_log_debug(0,"\t%s-check_hour():     Wrong hours: start hour is later than stop hour\n",
                 modname);
        free((void *) date1_s);
        free((void *) date2_s);
        return HOUR_FAIL;
    }
    else if ( (difftime(date2,clock) < 0.0) ||
              (difftime(clock,date1) < 0.0) )
    {
        lcas_log_debug(0,"\t%s-check_hour():     Hour (%02d:%02d:%02d) out of range: (%02d:%02d:%02d)-(%02d:%02d:%02d)\n",
            modname,
            clock_s->tm_hour,
            clock_s->tm_min,
            clock_s->tm_sec,
            date1_s->tm_hour,
            date1_s->tm_min,
            date1_s->tm_sec,
            date2_s->tm_hour,
            date2_s->tm_min,
            date2_s->tm_sec
        );
        free((void *) date1_s);
        free((void *) date2_s);
        return HOUR_FAIL;
    }

    free((void *) date1_s);
    free((void *) date2_s);
    return HOUR_OK;
}
