/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \page lcas_userban.mod banned users plugin

    \section userbansyn SYNOPSIS
        \b lcas_userban.mod [\<banned user file\>]

    \section userbandescr DESCRIPTION

        This plugin checks a file that contains a list of DNs (subjects of the X509 certificate) of users to be
        \e banned from the site.
        If the DN of the user for which the authorization request is made is found in the list, the plugin \e denies
        access to the site.

    \section userbanoptions OPTIONS
    \subsection userbanoptie1 [<banned user file>]
        The name of the file that contains the list of banned user DNs.
        Default search path: \c /opt/edg/lib/lcas/modules.

\section userbanReturnvalue RETURN VALUES
        \li LCAS_MOD_SUCCESS : Success
        \li LCAS_MOD_NOFILE  : the banned user file could not be found
        \li LCAS_MOD_FAIL    : Failure
 
\section userbanErrors ERRORS
        See bugzilla for known errors (http://marianne.in2p3.fr/datagrid/bugzilla/)
 
\section userbanSeeAlso SEE ALSO
        \ref lcas_userallow.mod "lcas_userallow.mod",
        \ref lcas_timeslots.mod "lcas_timeslots.mod",
        \ref lcas_voms.mod "lcas_voms.mod",
*/

/*!
    \file   lcas_userban.c
    \brief  Interface to the LCAS plugins
    \author Martijn Steenbakkers for the EU DataGrid.

    This file contains the code for the plugin that checks a plain text file containing a list
    of the DN (subjects of X509 certificates) of the users that are \e banned on the site.
    The interface consists of the following functions:
    -# plugin_initialize()
    -# plugin_confirm_authorization()
    -# plugin_terminate()
*/

/******************************************************************************

lcas_userban.c

Description:
    LCAS module that makes authorization decisions based
    on a ban list
    Currently it reads a plain file that contains the
    DN's of the banned users

CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/lcas-plugins-basic/src/userban/lcas_userban.c,v $
    $Date: 2010-05-03 10:44:00 $
    $Revision: 1.5 $
    $Author: okoeroo $

******************************************************************************/

/*****************************************************************************
                            Include header files
******************************************************************************/
#include "lcas_config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if HAVE_MALLOC_H
#include <malloc.h>
#endif

#include <lcas/lcas_modules.h>

#include "lcas_gridlist.h"
#include "paramextract.h"

/******************************************************************************
                       Define module specific variables
******************************************************************************/
static char *                 modname="lcas_userban.mod";
static char *                 userban_db = NULL;
static int                    disableWildCardMatching = 0;



/******************************************************************************
Function:   plugin_initialize
Description:
    Initialize plugin
Parameters:
    argc, argv
    argv[1]: database to be used by plugin
Returns:
    LCAS_MOD_SUCCESS : succes
    LCAS_MOD_FAIL    : failure
    LCAS_MOD_NOFILE  : db file not found
******************************************************************************/
#if 0
int plugin_initialize(char * plugin_db)
#endif
int plugin_initialize(int argc, char ** argv)
{
    int i;

    lcas_log_debug(2,"%s-plugin_initialize(): passed arguments:\n",modname);
    for (i=0; i < argc; i++)
    {
        lcas_log_debug(2,"%s-plugin_initialize(): arg %d is %s\n",
             modname,i,argv[i]);
    }

    disableWildCardMatching = isParamEnabled ("-disable_wildcards", argc, argv);

    disableWildCardMatching ? lcas_log_debug(2,"%s-plugin_initialize(): wildcard matching is disabled\n", modname)
                            : lcas_log_debug(2,"%s-plugin_initialize(): wildcard matching is enabled\n", modname);


    if (argc > 1)
        userban_db = lcas_findfile(argv[1]);

    /* Test if userban_db can be opened */
    if (userban_db == NULL)
    {
        lcas_log(0,"\t%s-plugin_initialize() error: banned user file required !\n",
                   modname);
        return LCAS_MOD_NOFILE;
    }
    if (lcas_getfexist(1,userban_db) == NULL)
    {
        lcas_log(0,
                 "\t%s-plugin_initialize() error: Cannot find banned user file: %s\n",
                 modname,userban_db
        );
        return LCAS_MOD_NOFILE;
    }
    return LCAS_MOD_SUCCESS;
}

/******************************************************************************
Function:   plugin_confirm_authorization
Description:
    Ask for authorization by passing RSL and user credential
Parameters:
    request:   RSL request
    user_cred: user credential
Returns:
    LCAS_MOD_SUCCESS: authorization succeeded
    LCAS_MOD_FAIL   : authorization failed
    LCAS_MOD_NOFILE : db file not found
******************************************************************************/
int
plugin_confirm_authorization(lcas_request_t request, lcas_cred_id_t lcas_cred)
{
    int                           rc;
    char *                        dummy = NULL;
    char *                        user_dn = NULL;

    /*
     * check credential and get the globus name
     */
    if ( (user_dn = lcas_get_dn(lcas_cred)) == NULL)
    {
        lcas_log(0, "lcas.mod-lcas_get_fabric_authorization() error: user DN empty\n");
        goto lcas_userban_noauth;
    }

    /* Do the check */
    lcas_log_debug(0,"\t%s-plugin_confirm_authorization(): checking banned users in %s\n",
             modname,userban_db);

    /* 
     * The new default is to perform the wildcard matching, based upon fnmatch 
     * You'll need to explicitly disable it in the initialization parameters
     */
    if (! disableWildCardMatching)
        rc = lcas_gridlist(user_dn, &dummy, userban_db, MATCH_WILD_CHARS|MATCH_ONLY_DN, NULL, NULL);
    else
        rc = lcas_gridlist(user_dn, &dummy, userban_db, MATCH_ONLY_DN, NULL, NULL);


    if ( rc == LCAS_MOD_ENTRY )
    {
        /* Entry found for user_dn, so the user is banned */
        lcas_log_debug(0,"\t%s-plugin_confirm_authorization(): entry found for %s\n",
                 modname,user_dn);
        goto lcas_userban_noauth;
    }
    else if ( rc == LCAS_MOD_NOFILE )
    {
        /* file not found */
        lcas_log(0,
            "\t%s-plugin_confirm_authorization() error: Cannot find banned user file: %s\n",
            modname,userban_db);
        goto lcas_userban_nofile;
    }

 lcas_userban_auth:
    /* authorization = no entry found for user_dn */
    if (dummy != NULL) free(dummy);
    return LCAS_MOD_SUCCESS;

 lcas_userban_noauth:
    /* no authorization = entry found for user_dn */
    if (dummy != NULL) free(dummy);
    return LCAS_MOD_FAIL;

 lcas_userban_nofile:
    /* file not found */
    if (dummy != NULL) free(dummy);
    return LCAS_MOD_NOFILE;
}

/******************************************************************************
Function:   plugin_terminate
Description:
    Terminate plugin
Parameters:

Returns:
    LCAS_MOD_SUCCESS : succes
    LCAS_MOD_FAIL    : failure
******************************************************************************/
int plugin_terminate()
{
    lcas_log_debug(1,"%s-plugin_terminate(): terminating\n",modname);
    if (userban_db) { free(userban_db); userban_db=NULL; }

    return LCAS_MOD_SUCCESS;
}
